/*
    MCP9600_4channel_INT_demo.ino
    Example for MCP9600

    Copyright (c) 2018 Seeed Technology Co., Ltd.
    Website    : www.seeed.cc
    Author     : downey
    Create Time: May 2018
    Change Log :

    The MIT License (MIT)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
*/

#include "Seeed_MCP9600.h"

#ifdef ARDUINO_SAMD_VARIANT_COMPLIANCE
    #define SERIAL SerialUSB
#else
    #define SERIAL Serial
#endif

MCP9600 sensor;

/** @brief interruption cfg.


 * */
err_t sensor_INT_config() {
    err_t ret = NO_ERROR;
    CHECK_RESULT(ret, sensor.set_filt_coefficients(FILT_MID));

    for (int i = 0; i < 4; i++) {
        /*Conver temp num to 16bit data*/
        CHECK_RESULT(ret, sensor.set_alert_limit(i, sensor.covert_temp_to_reg_form(28 + i)));
        /*  Set hysteresis.for example,set hysteresis to 2℃,when the INT limitation is 30℃,interruption will be generated when
            the temp ecceed limitation,and the interruption flag will stay unless the temp below 30-2(limitation-hysteresis) 28℃. */
        CHECK_RESULT(ret, sensor.set_alert_hys(i, 2));

        /*Set when interruption generated the pin's status*/
        CHECK_RESULT(ret, sensor.set_alert_bit(i, ACTIVE_LOW));

        CHECK_RESULT(ret, sensor.clear_int_flag(i));

        /*default is comparator mode*/
        CHECK_RESULT(ret, sensor.set_alert_mode_bit(i, COMPARE_MODE));

        /*Set alert pin ENABLE.*/
        CHECK_RESULT(ret, sensor.set_alert_enable(i, ENABLE));



    }

    /*device cfg*/
    CHECK_RESULT(ret, sensor.set_cold_junc_resolution(COLD_JUNC_RESOLUTION_0_25));
    CHECK_RESULT(ret, sensor.set_ADC_meas_resolution(ADC_14BIT_RESOLUTION));
    CHECK_RESULT(ret, sensor.set_burst_mode_samp(BURST_32_SAMPLE));
    CHECK_RESULT(ret, sensor.set_sensor_mode(NORMAL_OPERATION));

    return NO_ERROR;
}


err_t get_temperature(float* value) {
    err_t ret = NO_ERROR;
    float hot_junc = 0;
    float junc_delta = 0;
    float cold_junc = 0;
    bool stat = true;

    CHECK_RESULT(ret, sensor.check_data_update(&stat));
    if (stat) {
        CHECK_RESULT(ret, sensor.read_hot_junc(&hot_junc));
        CHECK_RESULT(ret, sensor.read_junc_temp_delta(&junc_delta));

        CHECK_RESULT(ret, sensor.read_cold_junc(&cold_junc));

        *value = hot_junc;
    } else {
        SERIAL.println("data not ready!!");
    }

    return NO_ERROR;
}


void setup() {
    SERIAL.begin(115200);
    delay(10);
    SERIAL.println("serial start!!");
    if (sensor.init(THER_TYPE_K)) {
        SERIAL.println("sensor init failed!!");
    }
    sensor_INT_config();
}



void loop() {
    float temp = 0;
    u8 byte = 0;
    u8 stat = 0;


    get_temperature(&temp);
    SERIAL.print("temperature ==============================>>");
    SERIAL.println(temp);

    sensor.read_INT_stat(&stat);

    SERIAL.println(" ");
    SERIAL.println(" ");

    delay(1000);
}